#! /usr/bin/rebol
REBOL[
	Title:		"OpenCV cxcore"
	Author:		"François Jouen"
	Rights:		"Copyright (c) 2012 François Jouen. All rights reserved."
	License: {
		Redistribution and use in source and binary forms, with or without modification,
		are permitted provided that the following conditions are met:

		    * Redistributions of source code must retain the above copyright notice,
		      this list of conditions and the following disclaimer.
		    * Redistributions in binary form must reproduce the above copyright notice,
		      this list of conditions and the following disclaimer in the documentation
		      and/or other materials provided with the distribution.

		THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
		ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
		FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
		DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
		SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
		CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
		OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
		OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	}
]

do %cxtypes.r
do %cxerror.r
do %cvver.r


libopencv: load/library %/Library/Frameworks/OpenCV.framework/Versions/A/OpenCV

;/****************************************************************************************\
;*          Array allocation, deallocation, initialization and access to elements         *
;\****************************************************************************************/

;/* <malloc> wrapper.
;If there is no enough memory, the function (as well as other OpenCV functions that call cvAlloc raises an error. */

ccvAlloc: make routine! [
	size    [integer!]
    return: [string!]
]libopencv "cvAlloc"

;/* <free> wrapper.

cvFree: make routine! [
	ptr		[integer!]
	return: [none!]
] libopencv "cvFree_"

;Rebol Specific ; normally we have to use a CvSize! structure to creates images or header
; but rebol returns a block of value so we used 2 integers (width and height)

;Allocates and initializes IplImage header
cvCreateImageHeader: make routine! compose/deep/only 
[
	width 		[integer!]
	height 		[integer!]
	depth		[integer!]
	channels    [integer!]
	return: [struct! (first IplImage!)] 
]libopencv "cvCreateImageHeader"

;Inializes IplImage header
cvInitImageHeader: make routine! compose/deep/only 
[
	image		[struct! (first IplImage!)]
	width 		[integer!]
	height 		[integer!]
	depth		[integer!]
	channels	[integer!]
	origin		[integer!];CV_DEFAULT(0
	align		[integer!];CV_DEFAULT(4)
	return: [struct! (first IplImage!)] 
]libopencv "cvInitImageHeader"

;Creates IPL image (header and data). can  be used by highgui!!!




cvCreateImage: make routine! compose/deep/only[
; create new image
	width 		[integer!]
	height 		[integer!]
	depth 		[integer!]
	channels 	[integer!]
	return: 	[struct! (first IplImage!)] ; returns an iplImage structure
] libopencv "cvCreateImage"

;Releases (i.e. deallocates) IPL image header
cvReleaseImageHeader: make routine! compose/deep/only[
	image		[struct! (first IplImage!)]
] libopencv "cvReleaseImageHeader"

;Releases IPL image header and data
cvReleaseImage: make routine! compose/deep/only[
	;image		[struct! (first IplImage!)]
	image      		[struct! (first struct*)]
] libopencv "cvReleaseImage"

;Creates a copy of IPL image (widthStep may differ) 
cvCloneImage: make routine! compose/deep/only[
	image		[struct! (first IplImage!)]
	return: [struct! (first IplImage!)] ; returns an iplImage structure
] libopencv "cvCloneImage"

;Sets a Channel Of Interest (only a few functions support COI) - use cvCopy to extract the selected channel and/or put it back
cvSetImageCOI: make routine! compose/deep/only[
	image		[struct! (first IplImage!)]
    coi			[integer!]
] libopencv "cvSetImageCOI"

;Retrieves image Channel Of Interest
cvGetImageCOI: make routine! compose/deep/only[
	image		[struct! (first IplImage!)]
	return:		[integer!]
] libopencv "cvGetImageCOI"

;Sets image ROI (region of interest) (COI is not changed)
cvSetImageROI: make routine! compose/deep/only[
	image		[struct! (first IplImage!)]
	rect		[struct! (first CvRect!)]
]libopencv "cvSetImageROI"

;Resets image ROI and COI */
cvResetImageROI: make routine! compose/deep/only[
	image		[struct! (first IplImage!)]
] libopencv "cvResetImageROI"

;Retrieves image ROI
cvGetImageROI: make routine! compose/deep/only[
	image		[struct! (first IplImage!)] 
	return: 	[struct! (first CvRect!)]
] libopencv "cvGetImageROI" 

;Allocates and initalizes CvMat header
cvCreateMatHeader: make routine! compose/deep/only
[ 
	rows	[integer!]
	cols	[integer!]
	type	[integer!]
	return:	[struct! (first CvMat!)]
]libopencv "cvCreateMatHeader"

CV_AUTOSTEP:  #7fffffff

;Initializes CvMat header
cvInitMatHeader: make routine! compose/deep/only
[ 
	mat		[struct! (first CvMat!)]
	rows	[integer!]
	cols	[integer!]
	type	[integer!]
	data    [integer!] ; null 
	step	[integer!] ; CV_DEFAULT(CV_AUTOSTEP
] libopencv "cvInitMatHeader"

;Allocates and initializes CvMat header and allocates data */
  
cvCreateMat: make routine! compose/deep/only [
 rows		[integer!]
 cols		[integer!]
 type 		[integer!]
 return:	[struct! (first CvMat!)]
]libopencv "cvCreateMat" 

;Releases CvMat header and deallocates matrix data (reference counting is used for data)
cvReleaseMat: make routine! compose/deep/only [
	mat		 [struct! (first CvMat!)]
] libopencv "cvReleaseMat"

;/* Decrements CvMat data reference counter and deallocates the data if it reaches
; inline function, not included in library. 
;use pointer to CvMat! To be tested with Rebol !!!!

cvDecRefData: func  [mat]
[
    if  (CV_IS_MAT mat)
    [
        mat/data/ptr: none
        if (mat/refcount != none  and mat/refcount = 0) [cvFree [mat/refcount]
        mat/refcount: none]
    ] 
    if (CV_IS_MATND mat)
    [
        mat/data/ptr: none
        if (mat/refcount != none  and mat/refcount == 0) [cvFree [mat/refcount]
        mat/refcount: none]
    ]
]

;Increments CvMat data reference counter */
cvIncRefData: func  [mat]
[
    refcount: 0
    if  (CV_IS_MAT mat)
    [
        if (mat/refcount != none) [refcount: refcount + 1 mat/refcount: refcount]
    ]
    if (CV_IS_MATND mat)
    [
       if (mat/refcount != none) [refcount: refcount + 1 mat/refcount: refcount]
    ]
    return refcount
]

;Creates an exact copy of the input matrix (except, may be, step value)
cvCloneMat: make routine! compose/deep/only [
	mat		[struct! (first CvMat!)]
	return:	[struct! (first CvMat!)]
] libopencv "cvCloneMat"

;* Makes a new matrix from <rect> subrectangle of input array No data is copied
cvGetSubRect: make routine! compose/deep/only [
	arr		[struct! (first CvMat!)] ; cvArr* : pointer to generic array; integer?
	submat	[struct! (first CvMat!)]
	rect	[struct! (first CvRect!)]
	return:	[struct! (first CvMat!)]
]libopencv "cvGetSubRect"

alias 'cvGetSubRect "cvGetSubArr"


;Selects row span of the input array: arr(start_row:delta_row:end_row (end_row is not included into the span)
;CVAPI(CvMat*) 

cvGetRows: make routine! compose/deep/only [
	arr			[struct! (first CvMat!)] ; cvArr* : pointer to generic array; integer?
	submat		[struct! (first CvMat!)]
	start_row   [integer!]
	end_row		[integer!]
	delta_row	[integer!] ; CV_DEFAULT(1)
	return:	[struct! (first CvMat!)]
]libopencv "cvGetRows"

; inline function
cvGetRow: func [arr submat row] [return cvGetRows arr submat row row + 1 1]

;Selects column span of the input array: arr(:,start_col:end_col) (end_col is not included into the span)
cvGetCols: make routine! compose/deep/only [
	arr			[struct! (first CvMat!)] ; cvArr* : pointer to generic array; integer?
	submat		[struct! (first CvMat!)]
	start_col	[integer!]
	end_col		[integer!]
	return:	[struct! (first CvMat!)]
] libopencv "cvGetCols"

; inline function
cvGetCol:func [arr submat col] [return cvGetCols arr submat col col + 1]

;Select a diagonal of the input array.
;(diag = 0 means the main diagonal, >0 means a diagonal above the main one,
;<0 - below the main one).
;The diagonal will be represented as a column (nx1 matrix).

cvGetDiag: make routine! compose/deep/only [
	arr			[struct! (first CvMat!)] ; cvArr* : pointer to generic array; integer?
	submat		[struct! (first CvMat!)]
	diag		[integer!];CV_DEFAULT(0)
	return:	[struct! (first CvMat!)]
]libopencv "cvGetDiag"

;low-level scalar <-> raw data conversion functions
cvScalarToRawData: make routine! compose/deep/only [
	scalar			[struct! (first CvScalar!)]	
	data			[integer!]; *void
	type			[integer!]
	extend_to_12	[integer!];CV_DEFAULT(0)
] libopencv "cvScalarToRawData"

cvRawDataToScalar: make routine! compose/deep/only [
	data			[integer!]; *void
	type			[integer!]
	scalar			[struct! (first CvScalar!)]
] libopencv "cvRawDataToScalar"

;Allocates and initializes CvMatND header
cvCreateMatNDHeader: make routine! compose/deep/only [
	dims		[integer!]
	sizes		[integer!]
	type		[integer!]
	return:		[struct! (first CvMatND!)]
] libopencv"cvCreateMatNDHeader"

;Allocates and initializes CvMatND header and allocates data
cvCreateMatND: make routine! compose/deep/only [
	dims		[integer!]
	sizes		[integer!]
	type		[integer!]
	return:		[struct! (first CvMatND!)]
]libopencv "cvCreateMatND"

;Initializes preallocated CvMatND header
cvInitMatNDHeader: make routine! compose/deep/only [
	mat			[struct! (first CvMatND!)]
	dims		[integer!]
	sizes		[integer!]
	type		[integer!]
	data		[integer!]; *void  CV_DEFAULT(NULL)
	return:		[struct! (first CvMatND!)]
]libopencv "cvInitMatNDHeader"

;inline Releases CvMatND: use CvMatND** mat as parameter
cvReleaseMatND: func [mat] [cvReleaseMat mat]                                   

;Creates a copy of CvMatND (except, may be, steps) */
cvCloneMatND: make routine! compose/deep/only [
	mat		[struct! (first CvMatND!)]
	return:	[struct! (first CvMatND!)]
] libopencv "cvCloneMatND"

;Allocates and initializes CvSparseMat header and allocates data */
cvCreateSparseMat: make routine! compose/deep/only[
	dims		[integer!]
	sizes		[integer!]
	type		[integer!]
	return:		[struct! (first CvSparseMat!)]
]libopencv "cvCreateSparseMat"
                                    
;Releases CvSparseMat
cvReleaseSparseMat: make routine! compose/deep/only [
	mat [struct! (first CvSparseMat!)] 
]libopencv "cvReleaseSparseMat"

;Creates a copy of CvSparseMat (except, may be, zero items) */
cvCloneSparseMat: make routine! compose/deep/only [
	mat 	[struct! (first CvSparseMat!)]
	return: [struct! (first CvSparseMat!)]
]libopencv "cvCloneSparseMat"

;Initializes sparse array iterator (returns the first node or NULL if the array is empty)
cvInitSparseMatIterator: make routine! compose/deep/only [
	mat 			[struct! (first CvSparseMat!)]
	mat_iterator	[struct! (first CvSparseMatIterator!)]
	return: 		[struct! (first CvSparseNode!)]
]libopencv "cvInitSparseMatIterator"


;returns next sparse array node (or NULL if there is no more nodes)
;function uses CvSparseMatIterator! as parameter and returns  CvSparseNode! 

cvGetNextSparseNode: func [mat_iterator]
[
    either (mat_iterator/node/_next) [return mat_iterator/node: mat_iterator/node/_next]
    [
        idx: 0;
        for idx mat_iterator/curidx mat_iterator/mat/hashsize 1
        [
            node:  mat_iterator/mat/hashtable[idx]
            if (node)
            [
                mat_iterator/curidx: idx
                return [mat_iterator/node: node]
            ]
        ]
        return none;
    ]
]

;**************** matrix iterator: used for n-ari operations on dense arrays *********
CV_MAX_ARR: 10

CvNArrayIterator!: make struct! compose/deep/only
[
    count				[integer!]		; number of arrays
    dims				[integer!]      ; number of dimensions to iterate
    size				[struct! (first CvSize!)]; maximal common linear size: { width = size, height = 1 }
    ptr					[integer!]		;pointers to the array slices [CV_MAX_ARR]
    stack				[integer!]		; for internal use [CV_MAX_DIM
    hdr					[integer!]		;/* pointers to the headers of the CvMatND! matrices that are processed [CV_MAX_ARR]
    									; may be we need a CvMatND! struct!  
] none

CV_NO_DEPTH_CHECK:     1
CV_NO_CN_CHECK:        2
CV_NO_SIZE_CHECK:      4

;initializes iterator that traverses through several arrays simulteneously
;(the function together with cvNextArraySlice is used for N-ari element-wise operations)
cvInitNArrayIterator: make routine! compose/deep/only [
	count			[integer!]
	arrs			[struct! (first CvArr*)]
	mask			[struct! (first CvArr*)]
	stubs			[struct! (first CvMatND!)]
	array_iterator	[struct! (first CvNArrayIterator!)]
	flags			[integer!]
	return:			[integer!] 
] libopencv"cvInitNArrayIterator"

;returns zero value if iteration is finished, non-zero (slice length) otherwise 
cvNextNArraySlice: make routine! compose/deep/only [
	array_iterator	[struct! (first CvNArrayIterator!)]
	return:			[integer!] 
] libopencv "cvNextNArraySlice"

;Returns type of array elements: CV_8UC1 ... CV_64FC4 ... 
cvGetElemType: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	return:		[integer!]
] libopencv "cvGetElemType"

;Retrieves number of an array dimensions and optionally sizes of the dimensions
cvGetDims: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	sizes		[integer!]; CV_DEFAULT(NULL)
	return:		[integer!]
] libopencv "cvGetDims"

;Retrieves size of a particular array dimension.
;For 2d arrays cvGetDimSize(arr,0) returns number of rows (image height) and cvGetDimSize(arr,1) returns number of columns (image width) */
cvGetDimSize: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	index		[integer!]; CV_DEFAULT(NULL)
	return:		[integer!]
] libopencv "cvGetDimSize"

;ptr = &arr(idx0,idx1,...). All indexes are zero-based, the major dimensions go first (e.g. (y,x) for 2D, (z,y,x) for 3D
cvPtr1D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]; 
	type		[integer!]; CV_DEFAULT(NULL)
	return:		[struct! (first int*)] ; or uchar*
] libopencv "cvPtr1D"

cvPtr2D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]; 
	idx1		[integer!];
	type		[integer!]; CV_DEFAULT(NULL)
	return:		[struct! (first int*)] ; or uchar*
] libopencv "cvPtr2D"

cvPtr3D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]; 
	idx1		[integer!];
	idx2		[integer!];
	type		[integer!]; CV_DEFAULT(NULL)
	return:		[struct! (first int*)] ; or uchar*
] libopencv "cvPtr3D"

;For CvMat or IplImage number of indices should be 2
;(row index (y) goes first, column index (x) goes next).
;For CvMatND or CvSparseMat number of infices should match number of <dims> and
;indices order should match the array dimension order

cvPtrND: make routine! compose/deep/only [
	arr					[struct! (first CvArr*)]
	idx					[integer!]; 
	type				[integer!];CV_DEFAULT(NULL)
	create_node			[integer!];CV_DEFAULT(1)
	precalc_hashval		[integer!]; CV_DEFAULT(NULL)
	return:		[struct! (first int*)] ; or uchar*
] libopencv "cvPtrND"

;value = arr(idx0,idx1,...)
cvGet1D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	return:		[struct! (first CvScalar!)]		
] libopencv "cvGet1D"

cvGet2D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!];
	return:		[struct! (first CvScalar!)]		
] libopencv "cvGet2D"

cvGet3D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!];
	idx2		[integer!];
	return:		[struct! (first CvScalar!)]		
] libopencv "cvGet3D"

cvGetND: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx		[integer!]
	return:		[struct! (first CvScalar!)]		
] libopencv "cvGetND"

;for 1-channel arrays 

cvGetReal1D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	return:		[decimal!]		
] libopencv "cvGetReal1D"

cvGetReal2D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!]
	return:		[decimal!]		
] libopencv "cvGetReal2D"

cvGetReal3D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!]
	idx2		[integer!]
	return:		[decimal!]	
] libopencv "cvGetReal3D"

cvGetRealND: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx			[integer!]
	return:		[decimal!]			
] libopencv "cvGetRealND"

;arr(idx0,idx1,...) = value
cvSet1D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	value		[struct! (first CvScalar!)]
] libopencv "cvSet1D"

cvSet2D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!]
	value		[struct! (first CvScalar!)]
] libopencv "cvSet2D"

cvSet3D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!]
	idx2		[integer!]
	value		[struct! (first CvScalar!)]
] libopencv "cvSet3D"

cvSetND: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx 		[integer!]
	value		[struct! (first CvScalar!)]
] libopencv "cvSetND"

;for 1-channel arrays
cvSetReal1D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	value		[decimal!]
] libopencv "cvSetReal1D"

cvSetReal2D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!]
	value		[decimal!]
] libopencv "cvSetReal2D"

cvSetReal3D: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx0		[integer!]
	idx1		[integer!]
	idx2		[integer!]
	value		[decimal!]
] libopencv "cvSetReal3D"

cvSetRealND: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx 		[integer!]
	value		[decimal!]
] libopencv "cvSetRealND"

;clears element of ND dense array, in case of sparse arrays it deletes the specified node
cvClearND: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	idx 		[integer!]
] libopencv "cvClearND"

{Converts CvArr (IplImage or CvMat,...) to CvMat.
If the last parameter is non-zero, function can
convert multi(>2)-dimensional array to CvMat as long as
the last array's dimension is continous. The resultant
matrix will be have appropriate (a huge) number of rows}
cvGetMat: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	header		[struct! (first CvMat!)]
	coi			[integer!];CV_DEFAULT(NULL)
	allowND		[integer!];CV_DEFAULT(0)
	return: 	[struct! (first CvMat!)]
]libopencv "cvGetMat"

; Converts CvArr (IplImage or CvMat) to IplImage
cvGetImage: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
    image_header 	[struct! (first IplImage!)]
	return: 		[struct! (first IplImage!)]	
] libopencv "cvGetImage"

{Changes a shape of multi-dimensional array.
new_cn == 0 means that number of channels remains unchanged.
new_dims == 0 means that number and sizes of dimensions remain the same
(unless they need to be changed to set the new number of channels)
if new_dims == 1, there is no need to specify new dimension sizes
The resultant configuration should be achievable w/o data copying.
If the resultant array is sparse, CvSparseMat header should be passed
to the function else if the result is 1 or 2 dimensional,
CvMat header should be passed to the function
else CvMatND header should be passed}

cvReshapeMatND: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
	sizeof_header	[integer!]
	header			[struct! (first CvArr*)]
	new_cn			[integer!]
	new_dims		[integer!]
	new_sizes		[integer!]
	return: 		[struct! (first CvArr*)]
]libopencv "cvReshapeMatND"

cvReshapeND: func [arr header new_cn new_dims new_sizes]
[ cvReshapeMatND arr size? header header new_cn new_dims new_sizes]

cvReshape: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
	header			[struct! (first CvArr*)]
	new_cn			[integer!]
	new_rows		[integer!];CV_DEFAULT(0)
	return: 		[struct! (first CvMat!)]
] libopencv "cvReshape"

;Repeats source 2d array several times in both horizontal and vertical direction to fill destination array */
cvRepeat: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
] libopencv "cvRepeat" 

;Allocates array data */
cvCreateData: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
] libopencv "cvCreateData"

;
;Releases array data 
cvReleaseData: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
] libopencv "cvReleaseData"

{Attaches user data to the array header. The step is reffered to
the pre-last dimension. That is, all the planes of the array
must be joint (w/o gaps)}
cvSetData: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
	data			[integer!];void* pointer 
	step			[integer!]
] libopencv "cvSetData"

{Retrieves raw data of CvMat, IplImage or CvMatND.
In the latter case the function raises an error if
the array can not be represented as a matrix}
cvGetRawData: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
	data			[integer!];uchar** pointer
	step			[struct! (first int*)]; int* CV_DEFAULT(NULL)
	roi_size		[struct! (first CvSize!)];CV_DEFAULT(NULL)
] libopencv "cvGetRawData"

;Returns width and height of array in elements
cvGetSize: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
	return:			[struct! (first CvSize!)]
] libopencv "cvGetSize"

;Copies source array to destination array 
cvCopy: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
	mask			[struct! (first CvArr*)]; CV_DEFAULT(NULL)
] libopencv "cvCopy"

;Sets all or "masked" elements of input array to the same value
cvSet: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
	value			[struct! (first CvScalar!)]
	mask			[struct! (first CvArr*)] ; CV_DEFAULT(NULL) 
] libopencv "cvSet"

;Clears all the array elements (sets them to 0) */
cvSetZero: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
] libopencv "cvSetZero"

alias 'cvSetZero "cvZero"

;Splits a multi-channel array into the set of single-channel arrays or extracts particular [color] plane
cvSplit: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst0			[struct! (first CvArr*)]
	dst1			[struct! (first CvArr*)]
	dst2			[struct! (first CvArr*)]
	dst3			[struct! (first CvArr*)]
] libopencv "cvSplit"

;Merges a set of single-channel arrays into the single multi-channel array
;or inserts one particular [color] plane to the array */
cvMerge: make routine! compose/deep/only [
	src0			[struct! (first CvArr*)]
	src1			[struct! (first CvArr*)]
	src2			[struct! (first CvArr*)]
	src3			[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
] libopencv "cvMerge"

;Copies several channels from input arrays to certain channels of output arrays
cvMixChannels: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	src_count	[integer!]
	dst			[struct! (first CvArr*)]
	dst_count	[integer!]
	from_to		[integer!]
	pair_count 	[integer!]	
]libopencv "cvMixChannels"

{Performs linear transformation on every source array element:
dst(x,y,c) = scale*src(x,y,c)+shift.
Arbitrary combination of input and output array depths are allowed
(number of channels must be the same), thus the function can be used
for type conversion}
cvConvertScale: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	scale		[decimal!];CV_DEFAULT(1)
	_shift		[decimal!];CV_DEFAULT(0)
] libopencv "cvConvertScale"

alias 'cvConvertScale "cvCvtScale"
alias 'cvConvertScale "cvScale"
cvConvert: func [src dst] [cvConvertScale src dst 1 0]

{Performs linear transformation on every source array element,
stores absolute value of the result:
dst(x,y,c) = abs(scale*src(x,y,c)+shift).
destination array must have 8u type.
In other cases one may use cvConvertScale + cvAbsDiffS}

cvConvertScaleAbs: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	scale		[decimal!];CV_DEFAULT(1)
	_shift		[decimal!];CV_DEFAULT(0)

] libopencv "cvConvertScaleAbs"

alias 'cvConvertScaleAbs "cvCvtScaleAbs"

;checks termination criteria validity and sets eps to default_eps (if it is not set),
;max_iter to default_max_iters (if it is not set)
cvCheckTermCriteria: make routine! compose/deep/only [
	criteria			[struct! (first CvTermCriteria!)]
	default_eps			[decimal!]
	default_max_iters 	[integer!]
	return: 			[struct! (first CvTermCriteria!)]
	
] libopencv "cvCheckTermCriteria"

;****************************************************************************************\
;*                   Arithmetic, logic and comparison operations                          *
;****************************************************************************************/
;dst(mask) = src1(mask) + src2(mask)

cvAdd: make routine! compose/deep/only [
	src1			[struct! (first CvArr*)]
	src2			[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
	mask			[struct! (first CvArr*)];CV_DEFAULT(NULL)
]libopencv "cvAdd"

;dst(mask) = src(mask) + value */
cvAddS: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	value			[struct! (first CvScalar!)]
	dst				[struct! (first CvArr*)]
	mask			[struct! (first CvArr*)];CV_DEFAULT(NULL)
]libopencv "cvAddS"

;dst(mask) = src1(mask) - src2(mask) */
cvSub: make routine! compose/deep/only [
	src1			[struct! (first CvArr*)]
	src2			[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
	mask			[struct! (first CvArr*)];CV_DEFAULT(NULL)
]libopencv "cvSub"

; dst(mask) = src(mask) - value = src(mask) + (-value) 
; CV_INLINE  void  cvSubS( const CvArr* src, CvScalar value, CvArr* dst, const CvArr* mask CV_DEFAULT(NULL))
cvSubS: func [src value dst mask]
[cvAddS src negate value/O negate value/1 negate value/2 negate value/3 dst mask]

;dst(mask) = value - src(mask)
cvSubRS: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	value			[struct! (first CvScalar!)]
	dst				[struct! (first CvArr*)]
	mask			[struct! (first CvArr*)];CV_DEFAULT(NULL)
]libopencv "cvSubRS"

;dst(idx) = src1(idx) * src2(idx) * scale (scaled element-wise multiplication of 2 arrays)
cvMul: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst 				[struct! (first CvArr*)]
	scale				[decimal!] ;CV_DEFAULT(1))
] libopencv "cvMul"

{element-wise division/inversion with scaling: 
dst(idx) = src1(idx) * scale / src2(idx)
or dst(idx) = scale / src2(idx) if src1 == 0}
    
cvDiv: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst 				[struct! (first CvArr*)]
	scale				[decimal!] ;CV_DEFAULT(1))
] libopencv "cvDiv"

;dst = src1 * scale + src2 */
cvScaleAdd: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	scale				[struct! (first CvScalar!)]
	src2				[struct! (first CvArr*)]
	dst 				[struct! (first CvArr*)]
]libopencv "cvScaleAdd"

cvAXPY: func [A real_scalar B C] [cvScaleAdd A real_scalar B C]

; dst = src1 * alpha + src2 * beta + gamma
cvAddWeighted: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	alpha				[decimal!]
	src2				[struct! (first CvArr*)]
	beta				[decimal!]
	gamma				[decimal!]
	dst					[struct! (first CvArr*)]
] libopencv "cvAddWeighted"

;result = sum_i(src1(i) * src2(i)) (results for all channels are accumulated together)
cvDotProduct: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	return:				[decimal!]
] libopencv "cvDotProduct"

;dst(idx) = src1(idx) & src2(idx)
cvAnd: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst 				[struct! (first CvArr*)]
	mask				[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvAnd" 

;dst(idx) = src(idx) & value */
cvAndS: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	value				[struct! (first CvScalar!)]
	dst 				[struct! (first CvArr*)]
	mask				[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvAndS" 

;dst(idx) = src1(idx) | src2(idx)
cvOr: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst 				[struct! (first CvArr*)]
	mask				[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvOr" 

;dst(idx) = src(idx) | value
cvOrS: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	value				[struct! (first CvScalar!)]
	dst 				[struct! (first CvArr*)]
	mask				[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvOrS" 

;dst(idx) = src1(idx) ^ src2(idx)
cvXor: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst 				[struct! (first CvArr*)]
	mask				[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvXor" 

;dst(idx) = src(idx) ^ value
cvXorS: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	value				[struct! (first CvScalar!)]
	dst 				[struct! (first CvArr*)]
	mask				[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvXorS" 

;/* dst(idx) = ~src(idx)
cvNot: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	dst					[struct! (first CvArr*)]
] libopencv "cvNot"

;dst(idx) = lower(idx) <= src(idx) < upper(idx) */
cvInRange: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	lower				[struct! (first CvArr*)]
	upper				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
] libopencv "cvInRange"

;dst(idx) = lower <= src(idx) < upper */
cvInRangeS: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	lower				[struct! (first CvScalar!)]
	upper				[struct! (first CvScalar!)]
	dst					[struct! (first CvArr*)]
] libopencv "cvInRangeS"

 CV_CMP_EQ:   0
 CV_CMP_GT:   1
 CV_CMP_GE:   2
 CV_CMP_LT:   3
 CV_CMP_LE:   4
 CV_CMP_NE:   5
 
 ;The comparison operation support single-channel arrays only. Destination image should be 8uC1 or 8sC1
 ;dst(idx) = src1(idx) _cmp_op_ src2(idx) */
 cvCmp: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst					[struct! (first CvArr*)]
	cmp_op				[integer!]
 ] libopencv "cvCmp"
 
;dst(idx) = src1(idx) _cmp_op_ value
 cvCmpS: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	value				[decimal!]
	dst					[struct! (first CvArr*)]
	cmp_op				[integer!]
 ] libopencv "cvCmpS"

;dst(idx) = min(src1(idx),src2(idx)
cvMin: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst					[struct! (first CvArr*)]
 ] libopencv "cvMin"

;dst(idx) = max(src1(idx),src2(idx))
cvMax: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst					[struct! (first CvArr*)]
 ] libopencv "cvMax"

;dst(idx) = min(src(idx),value)
 cvMinS: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	value				[decimal!]
	dst					[struct! (first CvArr*)]
 ] libopencv "cvMinS"
; dst(idx) = max(src(idx),value)
cvMaxS: make routine! compose/deep/only [
	src					[struct! (first CvArr*)]
	value				[decimal!]
	dst					[struct! (first CvArr*)]
 ] libopencv "cvMaxS"
 
 ; dst(x,y,c) = abs(src1(x,y,c) - src2(x,y,c))
 cvAbsDiff: make routine! compose/deep/only [
	src1				[struct! (first CvArr*)]
	src2				[struct! (first CvArr*)]
	dst					[struct! (first CvArr*)]
 ] libopencv "cvAbsDiff"
 
 ;dst(x,y,c) = abs(src(x,y,c) - value(c)) */
 cvAbsDiffS: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	value				[struct! (first CvScalar!)]
	dst					[struct! (first CvArr*)]
 ] libopencv "cvAbsDiffS"
 
 
cvAbs: func [src dst] [cvAbsDiffS src dst cvScalarAll 0]

;/****************************************************************************************\
;*                                Math operations                                         *
;\****************************************************************************************

 ;Does cartesian->polar coordinates conversion. Either of output components (magnitude or angle) is optional
 cvCartToPolar: make routine! compose/deep/only [
	x						[struct! (first CvArr*)]
	y						[struct! (first CvArr*)]
	magnitude				[struct! (first CvArr*)]
	angle					[struct! (first CvArr*)]; CV_DEFAULT(NULL)
	angle_in_degrees		[integer!]; CV_DEFAULT(0)
 ] libopencv "cvCartToPolar" 
 
{Does polar->cartesian coordinates conversion.
Either of output components (magnitude or angle) is optional.
If magnitude is missing it is assumed to be all 1's }
cvPolarToCart: make routine! compose/deep/only [
	magnitude				[struct! (first CvArr*)]
	angle					[struct! (first CvArr*)]; CV_DEFAULT(NULL)
	x						[struct! (first CvArr*)]
	y						[struct! (first CvArr*)]
	angle_in_degrees		[integer!]; CV_DEFAULT(0)
 ] libopencv "cvPolarToCart" 
 
;Does powering: dst(idx) = src(idx)^power */
cvPow: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
	_power			[decimal!]
] libopencv "cvPow"

{Does exponention: dst(idx) = exp(src(idx)).
Overflow is not handled yet. Underflow is handled.
Maximal relative error is ~7e-6 for single-precision input}

cvExp: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
] libopencv "cvExp"

{Calculates natural logarithms: dst(idx) = log(abs(src(idx))).
Logarithm of 0 gives large negative number(~-700)
Maximal relative error is ~3e-7 for single-precision output}
cvLog: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
] libopencv "cvLog"

;Fast arctangent calculation */
cvFastArctan: make routine! [
	y	[decimal!]
	x	[decimal!]
] libopencv "cvFastArctan"
;Fast cubic root calculation
cvCbrt: make routine! [
	value	[decimal!]
] libopencv "cvCbrt" 

{Checks array values for NaNs, Infs or simply for too large numbers
   (if CV_CHECK_RANGE is set). If CV_CHECK_QUIET is set,
   no runtime errors is raised (function returns zero value in case of "bad" values).
   Otherwise cvError is called}
   
CV_CHECK_RANGE:    1
CV_CHECK_QUIET:    2

cvCheckArr: make routine! compose/deep/only [
	arr				[struct! (first CvArr*)]
	flags			[integer!] ;CV_DEFAULT(0)
	min_val 		[decimal!]; CV_DEFAULT(0) 
	max_val 		[decimal!];CV_DEFAULT(0))
	
] libopencv "cvCheckArr"	
   
alias 'cvCheckArr "cvCheckArray"

CV_RAND_UNI:      0
CV_RAND_NORMAL:   1



cvRandArr: make routine! compose/deep/only [
	rng			   	[struct! (first CvRNG*)] ; point to CvRNG funct
	arr				[struct! (first CvArr*)]
	dist_type		[integer!]
	param1			[struct! (first CvScalar!)]
	param2			[struct! (first CvScalar!)]
	
] libopencv "cvRandArr"

cvRandShuffle: make routine! compose/deep/only [
	mat				[struct! (first CvArr*)]		
	rng				[struct! (first CvRNG*)] ; point to CvRNG funct
	iter_factor		[decimal!]; CV_DEFAULT(1.0)
] libopencv "cvRandShuffle"

;Finds real roots of a cubic equation */
cvSolveCubic: make routine! compose/deep/only [
	coeffs			[struct! (first CvMat!)]
	roots			[struct! (first CvMat!)]
	return: 		[integer!]

] libopencv "cvSolveCubic"

;/****************************************************************************************\
;*                                Matrix operations                                       *
;\****************************************************************************************/

;* Calculates cross product of two 3d vectors */
cvCrossProduct: make routine! compose/deep/only [
	src1			[struct! (first CvArr*)]
	src2			[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
] libopencv "cvCrossProduct"

;Extended matrix transform:
;dst = alpha*op(A)*op(B) + beta*op(C), where op(X) is X or X^T */
cvGEMM: make routine! compose/deep/only [
	src1			[struct! (first CvArr*)]
	src2			[struct! (first CvArr*)]
	alpha			[decimal!]
	src3			[struct! (first CvArr*)]
	beta			[decimal!]
	dst				[struct! (first CvArr*)]
	tABC			[integer!];CV_DEFAULT(0)	
] libopencv "cvGEMM"

alias 'cvGEMM "vMatMulAddEx"

;Matrix transform: dst = A*B + C, C is optional 
cvMatMulAdd: func [src1 src2 src3 dst][cvGEMM src1 src2 1.0 src3 1.0 dst 0 ]
cvMatMul: func [src1 src2 dst] [cvMatMulAdd src1 src2 none dst]
CV_GEMM_A_T: 1
CV_GEMM_B_T: 2
CV_GEMM_C_T: 4

;transforms each element of source array and stores resultant vectors in destination array

cvTransform: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	transmat	[struct! (first CvMat!)]
	shiftvec	[struct! (first CvMat!)]
] libopencv "cvTransform"

alias 'cvTransform "cvMatMulAddS"

;Does perspective transform on every element of input array 
cvPerspectiveTransform: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	mat			[struct! (first CvMat!)]
] libopencv "cvPerspectiveTransform"

;Calculates (A-delta)*(A-delta)^T (order=0) or (A-delta)^T*(A-delta) (order=1)
cvMulTransposed: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	order		[integer!]
	delta		[struct! (first CvArr*)]
	scale		[decimal!];CV_DEFAULT(1.)
] libopencv "cvMulTransposed"

;Tranposes matrix. Square matrices can be transposed in-place */
cvTranspose: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
] libopencv "cvTranspose"

alias 'cvTranspose "cvT"

{Mirror array data around horizontal (flip=0),
vertical (flip=1) or both(flip=-1) axises:
cvFlip(src) flips images vertically and sequences horizontally (inplace)}

cvFlip: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)] ;CV_DEFAULT(NULL)
	flip_mode	[integer!]
] libopencv "cvFlip"

alias 'cvFlip "cvMirror"

CV_SVD_MODIFY_A:   1
CV_SVD_U_T:        2
CV_SVD_V_T:        4

;Performs Singular Value Decomposition of a matrix
cvSVD: make routine! compose/deep/only [
	A			[struct! (first CvArr*)]
	W			[struct! (first CvArr*)]
	U			[struct! (first CvArr*)]
	V			[struct! (first CvArr*)]
	flags		[integer!]
] libopencv "cvSVD"

;Performs Singular Value Back Substitution (solves A*X = B): flags must be the same as in cvSVD
cvSVBkSb: make routine! compose/deep/only [
	W			[struct! (first CvArr*)]
	U			[struct! (first CvArr*)]
	V			[struct! (first CvArr*)]
	B			[struct! (first CvArr*)]
	flags		[integer!]
] libopencv "cvSVBkSb"

CV_LU:  0
CV_SVD: 1
CV_SVD_SYM: 2

;Inverts matrix 
cvInvert: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	method		[integer!]
	return: 	[decimal!];CV_DEFAULT(CV_LU)
] libopencv "cvInvert"

alias 'cvInvert "cvInv"

;Solves linear system (src1)*(dst) = (src2) (returns 0 if src1 is a singular and CV_LU method is used)
cvSolve: make routine! compose/deep/only [
	src1		[struct! (first CvArr*)]
	src2		[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	method		[integer!];CV_DEFAULT(CV_LU)
	return: 	[integer!]
] libopencv "cvSolve"

;Calculates determinant of input matrix
cvDet: make routine! compose/deep/only [
	mat			[struct! (first CvArr*)]
	return: 	[decimal!]
] libopencv "cvDet"

;Calculates trace of the matrix (sum of elements on the main diagonal)
cvTrace: make routine! compose/deep/only [
	mat			[struct! (first CvArr*)]
	return: 	[struct! (first CvScalar!)]
] libopencv "cvTrace"

;Finds eigen values and vectors of a symmetric matrix
cvEigenVV: make routine! compose/deep/only [
	mat			[struct! (first CvArr*)]
	evects		[struct! (first CvArr*)]
	evals		[struct! (first CvArr*)]
	eps			[decimal!];CV_DEFAULT(0)
] libopencv "cvEigenVV"

;Makes an identity matrix (mat_ij = i == j)
cvSetIdentity: make routine! compose/deep/only [
	mat			[struct! (first CvArr*)]
	value		[struct! (first CvScalar!)] ;CV_DEFAULT(cvRealScalar 1
] libopencv "cvSetIdentity"

;Fills matrix with given range of numbers
cvRange: make routine! compose/deep/only [
	mat			[struct! (first CvMat!)]
	start		[decimal!]
	end			[decimal!]
] libopencv "cvRange"

; Calculates covariation matrix for a set of vectors transpose([v1-avg, v2-avg,...]) * [v1-avg,v2-avg,...] 
 CV_COVAR_SCRAMBLED: 0

; [v1-avg, v2-avg,...] * transpose([v1-avg,v2-avg,...]) 
 CV_COVAR_NORMAL:    1

; do not calc average (i.e. mean vector) - use the input vector instead (useful for calculating covariance matrix by parts) 
 CV_COVAR_USE_AVG:   2

; scale the covariance matrix coefficients by number of the vectors 
 CV_COVAR_SCALE:     4

; all the input vectors are stored in a single matrix, as its rows 
 CV_COVAR_ROWS:      8

; all the input vectors are stored in a single matrix, as its columns 
 CV_COVAR_COLS:     16

cvCalcCovarMatrix: make routine! compose/deep/only [
	vects		[struct! (first CvArr*)]
	count		[integer!]
	cov_mat		[struct! (first CvArr*)]
	avg			[struct! (first CvArr*)]
	flags		[integer!]
] libopencv "cvCalcCovarMatrix"

CV_PCA_DATA_AS_ROW: 	0 
CV_PCA_DATA_AS_COL: 	1
CV_PCA_USE_AVG:		 	2

cvCalcPCA: make routine! compose/deep/only [
	data		[struct! (first CvArr*)]
	mean		[struct! (first CvArr*)]
	eigenvals	[struct! (first CvArr*)]
	eigenvects	[struct! (first CvArr*)]
	flags		[integer!]
] libopencv "cvCalcPCA"
 
cvProjectPCA: make routine! compose/deep/only [
	data		[struct! (first CvArr*)]
	mean		[struct! (first CvArr*)]
	eigenvects	[struct! (first CvArr*)]
	result	[struct! (first CvArr*)]
] libopencv "cvProjectPCA"

cvBackProjectPCA: make routine! compose/deep/only [
	proj		[struct! (first CvArr*)]
	mean		[struct! (first CvArr*)]
	eigenvects	[struct! (first CvArr*)]
	result	[struct! (first CvArr*)]
] libopencv "cvBackProjectPCA"

;Calculates Mahalanobis(weighted) distance
cvMahalanobis: make routine! compose/deep/only [
	vec1		[struct! (first CvArr*)]
	vec2		[struct! (first CvArr*)]
	mat			[struct! (first CvArr*)]
	return: 	[decimal!]
] libopencv "cvMahalanobis"

alias 'cvMahalanobis "cvMahalonobis"

;/****************************************************************************************\
;*                                    Array Statistics                                    *
;\****************************************************************************************/

;Finds sum of array elements 
cvSum: make routine! compose/deep/only [
	arr		[struct! (first CvArr*)]
	return:	[struct! (first CvScalar!)]
] libopencv "cvSum"

;Calculates number of non-zero pixels 
cvCountNonZero: make routine! compose/deep/only [
	arr		[struct! (first CvArr*)]
	return:	[integer!]
] libopencv "cvCountNonZero"

;Calculates mean value of array elements */
cvAvg: make routine! compose/deep/only [
	arr		[struct! (first CvArr*)]
	mask	[struct! (first CvArr*)];CV_DEFAULT(NULL)
	return:	[struct! (first CvScalar!)]
] libopencv "cvAvg"

;Calculates mean and standard deviation of pixel values */
cvAvgSdv: make routine! compose/deep/only [
	arr		[struct! (first CvArr*)]
	mean	[struct! (first CvScalar!)]
	std_dev [struct! (first CvScalar!)]
	mask	[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvAvgSdv"

;Finds global minimum, maximum and their positions
cvMinMaxLoc: make routine! compose/deep/only [
	arr			[struct! (first CvArr*)]
	min_val		[decimal!]
	max_val		[decimal!]
	min_loc		[struct! (first CvPoint!)];CV_DEFAULT(NULL)
	max_loc		[struct! (first CvPoint!)];CV_DEFAULT(NULL)
	mask	 	[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvMinMaxLoc"

;types of array norm 
 CV_C:            1
 CV_L1:           2
 CV_L2:           4
 CV_NORM_MASK:    7
 CV_RELATIVE:     8
 CV_DIFF:         16
 CV_MINMAX:       32
 CV_DIFF_C:       CV_DIFF OR CV_C
 CV_DIFF_L1:      CV_DIFF OR CV_L1
 CV_DIFF_L2:      CV_DIFF OR CV_L2
 CV_RELATIVE_C:   CV_RELATIVE OR CV_C
 CV_RELATIVE_L1:  CV_RELATIVE OR CV_L1
 CV_RELATIVE_L2:  CV_RELATIVE OR CV_L2
 
 ;Finds norm, difference norm or relative difference norm for an array (or two arrays)
 cvNorm: make routine! compose/deep/only [
	arr1			[struct! (first CvArr*)]
	arr2			[struct! (first CvArr*)];CV_DEFAULT(NULL)
	norm_type 		[integer!]; CV_DEFAULT(CV_L2
	mask	 		[struct! (first CvArr*)];CV_DEFAULT(NULL)
	return:			[decimal!]
 ]libopencv "cvNorm"
 
cvNormalize: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	a			[decimal!];CV_DEFAULT(1.)
	b			[decimal!];CV_DEFAULT(0.)
	norm_type	[integer!];CV_DEFAULT(CV_L2)
	mask	 		[struct! (first CvArr*)];CV_DEFAULT(NULL)
] libopencv "cvNormalize"
 
CV_REDUCE_SUM: 0
CV_REDUCE_AVG: 1
CV_REDUCE_MAX: 2
CV_REDUCE_MIN: 3

cvReduce: make routine! compose/deep/only [
	src			[struct! (first CvArr*)]
	dst			[struct! (first CvArr*)]
	dim 		[integer!];CV_DEFAULT(-1)
	op 			[integer!];CV_DEFAULT(CV_REDUCE_SUM) )
] libopencv "cvReduce"

;/****************************************************************************************\
;*                      Discrete Linear Transforms and Related Functions                  *
;\****************************************************************************************/

 CV_DXT_FORWARD:  0
 CV_DXT_INVERSE:  1
 CV_DXT_SCALE:    2 ;divide result by size of array 
 CV_DXT_INV_SCALE: CV_DXT_INVERSE + CV_DXT_SCALE
 CV_DXT_INVERSE_SCALE: CV_DXT_INV_SCALE
 CV_DXT_ROWS:     4 ;transform each row individually 
 CV_DXT_MUL_CONJ: 8 ;conjugate the second argument of cvMulSpectrums 
 
 ;Discrete Fourier Transform: complex->complex,real->ccs (forward),ccs->real (inverse)
 cvDFT: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
	flags			[integer!]
	nonzero_rows 	[integer!];CV_DEFAULT(0)
] libopencv "cvDFT"

alias 'cvDFT "cvFFT"

;Multiply results of DFTs: DFT(X)*DFT(Y) or DFT(X)*conj(DFT(Y))
cvMulSpectrums: make routine! compose/deep/only [
	src1			[struct! (first CvArr*)]
	src2			[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
	flags			[integer!]
] libopencv "cvMulSpectrums"

;Finds optimal DFT vector size >= size0
cvGetOptimalDFTSize: make routine! [
	size0		[integer!]
	return:		[integer!]
] libopencv "cvGetOptimalDFTSize"

;Discrete Cosine Transform
cvDCT: make routine! compose/deep/only [
	src				[struct! (first CvArr*)]
	dst				[struct! (first CvArr*)]
	flags			[integer!]
] libopencv "cvDCT"

;/****************************************************************************************\
;*                              Dynamic data structures                                   *
;\****************************************************************************************/

;Calculates length of sequence slice (with support of negative indices)
cvSliceLength: make routine! compose/deep/only [
	slice		[struct! (first CvSlice!)]
	seq			[struct! (first CvSeq!)]
	return:		[integer!]
] libopencv "cvSliceLength"

;Creates new memory storage. block_size == 0 means that default,somewhat optimal size, is used (currently, it is 64K)
cvCreateMemStorage: make routine! compose/deep/only [
	block_size 		[integer!];CV_DEFAULT(0)
	return:			[struct! (first CvMemStorage!)]
] libopencv "cvCreateMemStorage"

;Creates a memory storage that will borrow memory blocks from parent storage
cvCreateChildMemStorage: make routine! compose/deep/only [
	parent	 		[struct! (first CvMemStorage!)]
	return:			[struct! (first CvMemStorage!)]
] libopencv "cvCreateChildMemStorage"

;Releases memory storage. All the children of a parent must be released before the parent. 
;A child storage returns all the blocks to parent when it is released

cvReleaseMemStorage: make routine! compose/deep/only [
	storage	 		[struct! (first CvMemStorage!)] ;CvMemStorage** (address?)
] libopencv "cvReleaseMemStorage"

{Clears memory storage. This is the only way(!!!) (besides cvRestoreMemStoragePos)
to reuse memory allocated for the storage - cvClearSeq,cvClearSet ...
do not free any memory.A child storage returns all the blocks to the parent when it is cleared}

cvClearMemStorage: make routine! compose/deep/only [
	storage	 		[struct! (first CvMemStorage!)] ;CvMemStorage** (address?)
] libopencv "cvClearMemStorage"

;Remember a storage "free memory" position 
cvSaveMemStoragePos: make routine! compose/deep/only [
	storage	 		[struct! (first CvMemStorage!)] 
	pos		 		[struct! (first CvMemStorage!)] 
] libopencv "cvSaveMemStoragePos"

;Restore a storage "free memory" position
cvRestoreMemStoragePos: make routine! compose/deep/only [
	storage	 		[struct! (first CvMemStorage!)] 
	pos		 		[struct! (first CvMemStorage!)] 
] libopencv "cvRestoreMemStoragePos"

;Allocates continuous buffer of the specified size in the storage */
cvMemStorageAlloc: make routine! compose/deep/only [
	storage	 		[struct! (first CvMemStorage!)] 
	size_t		 	[integer!] 
] libopencv "cvMemStorageAlloc"

;Allocates string in memory storage */
cvMemStorageAlloc: make routine! compose/deep/only [
	storage	 		[struct! (first CvMemStorage!)] 
	ptr		 	    [struct! (first char*)] ; pointer
	len				[integer!];CV_DEFAULT(-1)
	return:			[string!]
] libopencv "cvMemStorageAlloc"

;Creates new empty sequence that will reside in the specified storage 
cvCreateSeq: make routine! compose/deep/only [
	seq_flags		[integer!]	
	header_size		[integer!]
	elem_size		[integer!]
	storage			[struct! (first CvMemStorage!)]
	return: 		[struct! (first CvSeq!)]
] libopencv "cvCreateSeq"

;changes default size (granularity) of sequence blocks. The default size is ~1Kbyte
cvSetSeqBlockSize: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	delta_elems		[integer!]
] libopencv "cvSetSeqBlockSize"

;Adds new element to the end of sequence. Returns pointer to the element
cvSeqPush: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	element 		[integer!];CV_DEFAULT(NULL) pointer
	return: 		[struct! (first char*)]
] libopencv "cvSeqPush"

;Adds new element to the beginning of sequence. Returns pointer to it
cvSeqPushFront: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	element 		[integer!];CV_DEFAULT(NULL) pointer
	return: 		[struct! (first char*)]
] libopencv "cvSeqPushFront"

;Removes the last element from sequence and optionally saves it
cvSeqPop: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	element 		[integer!];CV_DEFAULT(NULL) pointer
] libopencv "cvSeqPop"

;Removes the first element from sequence and optioanally saves it
cvSeqPopFront: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	element 		[integer!];CV_DEFAULT(NULL) pointer
] libopencv "cvSeqPopFront"

CV_FRONT: 1
CV_BACK: 0

;Adds several new elements to the end of sequence
cvSeqPushMulti: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	element 		[integer!];CV_DEFAULT(NULL) pointer
	count			[integer!]
	in_front		[integer!]; CV_DEFAULT(0)
] libopencv "cvSeqPushMulti"

;Removes several elements from the end of sequence and optionally saves them
cvSeqPopMulti: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	element 		[integer!];CV_DEFAULT(NULL) pointer
	count			[integer!]
	in_front		[integer!]; CV_DEFAULT(0)
] libopencv "cvSeqPopMulti"

;Inserts a new element in the middle of sequence.cvSeqInsert(seq,0,elem) == cvSeqPushFront(seq,elem)
cvSeqInsert: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	before_index	[integer!]
	element 		[integer!];CV_DEFAULT(NULL) pointer
	return: 		[struct! (first char*)]
] libopencv "cvSeqInsert"

;Removes specified sequence element
cvSeqRemove: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	index			[integer!]
] libopencv "cvSeqRemove"

{Removes all the elements from the sequence. The freed memory
can be reused later only by the same sequence unless cvClearMemStorage
or cvRestoreMemStoragePos is called}
cvClearSeq: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
] libopencv "cvClearSeq"

{Retrives pointer to specified sequence element.
Negative indices are supported and mean counting from the end
(e.g -1 means the last sequence element)}
cvGetSeqElem: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	index			[integer!]
	return:			[struct! (first char*)]
] libopencv "cvGetSeqElem"

;Calculates index of the specified sequence element. Returns -1 if element does not belong to the sequence 
cvSeqElemIdx: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	element			[integer!] ; pointer to void*
	return:			[struct! (first CvSeqBlock!)]; CV_DEFAULT(NULL) address?
] libopencv "cvSeqElemIdx"

;Initializes sequence writer. The new elements will be added to the end of sequence
cvStartAppendToSeq: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	writer			[struct! (first CvSeqWriter!)]
] libopencv "cvStartAppendToSeq"

;Combination of cvCreateSeq and cvStartAppendToSeq
cvStartWriteSeq: make routine! compose/deep/only [
	seq_flags		[integer!]
	header_size		[integer!]
    elem_size		[integer!]
    storage			[struct! (first CvMemStorage!)]
    writer			[struct! (first CvSeqWriter!)]
] libopencv "cvStartWriteSeq"

{Closes sequence writer, updates sequence header and returns pointer to the resultant sequence
(which may be useful if the sequence was created using cvStartWriteSeq))}
cvEndWriteSeq: make routine! compose/deep/only [
	writer			[struct! (first CvSeqWriter!)]
	return: 		[struct! (first CvSeq!)]
] libopencv "cvEndWriteSeq"

; Updates sequence header. May be useful to get access to some of previously written elements via cvGetSeqElem or sequence reader
cvFlushSeqWriter: make routine! compose/deep/only [
	writer			[struct! (first CvSeqWriter!)]
] libopencv "cvFlushSeqWriter"

;Initializes sequence reader. The sequence can be read in forward or backward direction
cvStartReadSeq: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	reader			[struct! (first CvSeqReader!)]
	_reverse		[integer!];CV_DEFAULT(0)
] libopencv "cvStartReadSeq"

;Returns current sequence reader position (currently observed sequence element)
cvGetSeqReaderPos: make routine! compose/deep/only [
	reader			[struct! (first CvSeqReader!)]
	return:			[integer!]
] libopencv "cvGetSeqReaderPos"

;Changes sequence reader position. It may seek to an absolute or to relative to the current position
cvSetSeqReaderPos: make routine! compose/deep/only [
	reader			[struct! (first CvSeqReader!)]
	index			[integer!]
	is_relative 	[integer!];CV_DEFAULT(0))
] libopencv "cvSetSeqReaderPos"

;Copies sequence content to a continuous piece of memory 
cvCvtSeqToArray: make routine! compose/deep/only [
	seq				[struct! (first CvSeq!)]
	elements		[integer!]; pointer
	slice		 	[struct! (first CvSlice!)];CV_DEFAULT(CV_WHOLE_SEQ)
] libopencv "cvCvtSeqToArray"

;Creates sequence header for array.
;After that all the operations on sequences that do not alter the conten can be applied to the resultant sequence
cvMakeSeqHeaderForArray: make routine! compose/deep/only [
	seq_type			[struct! (first CvSeq!)]
	header_size		 	[integer!]
	elem_size		 	[integer!]
	elements			[integer!]; pointer
	total				[integer!]
	seq					[struct! (first CvSeq!)]
	block				[struct! (first CvSeqBlock!)]
	return:			[struct! (first CvSeq!)]
] libopencv "cvMakeSeqHeaderForArray"

;Extracts sequence slice (with or without copying sequence elements)
cvSeqSlice: make routine! compose/deep/only [
	seq					[struct! (first CvSeq!)]
	slice 				[struct! (first CvSlice!)]
	storage				[struct! (first CvMemStorage!)];CV_DEFAULT(NULL)
	copy_data			[integer!];CV_DEFAULT(NULL)
] libopencv "cvSeqSlice"

;inline function
cvCloneSeq: func [seq storage 1]
[
    return cvSeqSlice seq CV_WHOLE_SEQ storage 1
]

;Removes sequence slice
cvSeqRemoveSlice: make routine! compose/deep/only [
	seq					[struct! (first CvSeq!)]
	slice 				[struct! (first CvSlice!)]
] libopencv "cvSeqRemoveSlice"

;Inserts a sequence or array into another sequence
cvSeqInsertSlice: make routine! compose/deep/only [
	seq					[struct! (first CvSeq!)]
	before_index		[integer!]
	from_arr			[struct! (first CvArr*)]
] libopencv "cvSeqInsertSlice"

;a < b ? -1 : a > b ? 1 : 0 
CvCmpFunc: func [a b userdata][
	tmp: 0
	if a < b [tmp: -1]
	if a > b [tmp: 1]
	return tmp
]
CV_CDECL*: :CvCmpFunc

;Sorts sequence in-place given element comparison function
cvSeqSort: make routine! compose/deep/only [
	seq					[struct! (first CvSeq!)]
	CvCmpFunc			[integer!]
	userdata			[integer!];CV_DEFAULT(NULL)
] libopencv "cvSeqSort"

;Finds element in a [sorted] sequence 
cvSeqSearch: make routine! compose/deep/only [
	seq					[struct! (first CvSeq!)]
	elem				[integer!]
	CvCmpFunc			[integer!]
	is_sorted			[integer!]
	elem_idx			[struct! (first int*)]
	userdata			[integer!];CV_DEFAULT(NULL)
] libopencv "cvSeqSearch"

;Reverses order of sequence elements in-place
cvSeqInvert: make routine! compose/deep/only [
	seq					[struct! (first CvSeq!)]
] libopencv "cvSeqInvert"

;Splits sequence into one or more equivalence classes using the specified criteria
cvSeqPartition: make routine! compose/deep/only [
	seq					[struct! (first CvSeq!)]
	storage				[struct! (first CvMemStorage!)]
	labels				[struct! (first CvSeq!)]; CvSeq** 
	is_equal			[integer!]
	userdata			[integer!]
	return:				[integer!]
] libopencv "cvSeqPartition"

;/************ Internal sequence functions ************/
cvChangeSeqBlock: make routine![
	reader				[integer!]
	direction			[integer!]
] libopencv "cvChangeSeqBlock"

cvCreateSeqBlock: make routine! compose/deep/only [
	writer				[struct! (first CvSeqWriter!)]
] libopencv "cvCreateSeqBlock"

;Creates a new set
cvCreateSet: make routine! compose/deep/only [
	set_flags			[integer!]
	header_size			[integer!]
	elem_size			[integer!]
	storage				[struct! (first CvMemStorage!)]
	return:				[struct! (first CvSet!)]
] libopencv "cvCreateSet"

;Adds new element to the set and returns pointer to it
cvSetAdd: make routine! compose/deep/only [
	set_header			[struct! (first CvSet!)]
	elem			    [struct! (first CvSetElem!)]; V_DEFAULT(NULL)
	inserted_elem 		[struct! (first CvSetElem!)] ;CvSetElem** V_DEFAULT(NULL)
	return:				[integer!]
] libopencv "cvSetAdd"

;inline Fast variant of cvSetAdd; DO NOT USE WITH REBOL
cvSetNew: func [set_header]
[
	elem: set_header/free_elems;
    either (elem)
    [
        set_header/free_elems: elem/next_free
        elem/flags: elem/flags and CV_SET_ELEM_IDX_MASK
        ++ set_header/active_count
    ] [cvSetAdd set_header none &elem]; if rebol use address of elem (seek peek and poke func)
    return elem
]

;inline Removes set element given its pointer 
;REBOL !! use CvSet! as first parameter and CvSetElem! as second

cvSetRemoveByPtr: func [set_header elem]
[
    _elem:  make struct! [CvSetElem!] elem
     assert _elem-/flags >= 0 
    _elem/next_free: set_header/free_elems
    _elem/flags: (_elem/flags AND CV_SET_ELEM_IDX_MASK) OR CV_SET_ELEM_FREE_FLAG
    set_header/free_elems:_elem;
    -- set_header/active_count
]

;Removes element from the set by its index 
cvSetRemove: make routine! compose/deep/only [ 
	set_header		[struct! (first CvSet!)]
	index 			[integer!]
] libopencv "cvSetRemove"

;inline func Returns a set element by index. If the element doesn't belong to the set,NULL is returned
cvGetSetElem: func [set_header index ]
[
    elem: cvGetSeqElem set_header index
    tmp: elem AND CV_IS_SET_ELEM elem
    either tmp <> 0 [return tmp] [return 0];
]

;Removes all the elements from the set
cvClearSet: make routine! compose/deep/only [ 
	set_header		[struct! (first CvSet!)]
] libopencv "cvClearSet"

;Creates new graph
cvCreateGraph: make routine! compose/deep/only [
	graph_flags			[integer!]
	header_size			[integer!]
    vtx_size			[integer!]
    edge_size			[integer!]
    storage				[struct! (first CvMemStorage!)]
    return:				[struct! (first CvGraph!)]
] libopencv "cvCreateGraph"

;Adds new vertex to the graph
cvGraphAddVtx: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	vtx				[struct! (first CvGraphVtx!)];CV_DEFAULT(NULL)
	inserted_vtx	[struct! (first CvGraphVtx!)]; pointer address? CV_DEFAULT(NULL)
    return:			[integer!]
] libopencv "cvGraphAddVtx"

;Removes vertex from the graph together with all incident edges

cvGraphRemoveVtx: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	index			[integer!]
    return:			[integer!]
] libopencv "cvGraphRemoveVtx"

cvGraphRemoveVtxByPtr: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	vtx				[struct! (first CvGraphVtx!)];CV_DEFAULT(NULL)
    return:			[integer!]
] libopencv "cvGraphRemoveVtxByPtr"

{Link two vertices specifed by indices or pointers if they
are not connected or return pointer to already existing edge
connecting the vertices. Functions return 1 if a new edge was created, 0 otherwise}
cvGraphAddEdge: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	start_idx		[integer!]
	end_idx			[integer!]
	edge			[struct! (first CvGraphEdge!)];CV_DEFAULT(NULL)
	inserted_edge	[struct! (first CvGraphEdge!)];pointer address CV_DEFAULT(NULL)
    return:			[integer!]
] libopencv "cvGraphAddEdge"

cvGraphAddEdgeByPtr: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	start_vtx		[struct! (first CvGraphVtx!)]
	end_vtx			[struct! (first CvGraphVtx!)]
	edge			[struct! (first CvGraphEdge!)];CV_DEFAULT(NULL)
	inserted_edge	[struct! (first CvGraphEdge!)];pointer address CV_DEFAULT(NULL)
    return:			[integer!]
] libopencv "cvGraphAddEdgeByPtr"

;* Remove edge connecting two vertices
cvGraphRemoveEdge: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	start_idx		[integer!]
	end_idx			[integer!]
] libopencv "cvGraphRemoveEdge"

cvGraphRemoveEdgeByPtr: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	start_vtx		[struct! (first CvGraphVtx!)]
	end_vtx			[struct! (first CvGraphVtx!)]
] libopencv "cvGraphRemoveEdgeByPtr"

;* Find edge connecting two vertices
cvFindGraphEdge: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	start_idx		[integer!]
	end_idx			[integer!]
	return:			[struct! (first CvGraphEdge!)]
] libopencv "cvFindGraphEdge"

cvFindGraphEdgeByPtr: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	start_idx		[struct! (first CvGraphVtx!)]
	end_idx			[struct! (first CvGraphVtx!)]
	return:			[struct! (first CvGraphEdge!)]
] libopencv "cvFindGraphEdgeByPtr"

alias 'cvFindGraphEdge "cvGraphFindEdge"
alias 'cvFindGraphEdgeByPtr "cvGraphFindEdgeByPtr"

;Remove all vertices and edges from the graph 
cvClearGraph: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
] libopencv "cvClearGraph"

;Count number of edges incident to the vertex
cvGraphVtxDegree: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	vtx_idx			[integer!]
	return:			[integer!]
] libopencv "cvGraphVtxDegree"

cvGraphVtxDegreeByPtr: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	vtx				[struct! (first CvGraphVtx!)]
	return:			[integer!]
] libopencv "cvGraphVtxDegreeByPtr"

;Retrieves graph vertex by given index
cvGetGraphVtx: func [graph idx] [cvGetSetElem graph idx]
;Retrieves index of a graph vertex given its pointer
cvGraphVtxIdx: func [graph vtx] [vtx/flags AND CV_SET_ELEM_IDX_MASK]
;Retrieves index of a graph edge given its pointer */
cvGraphEdgeIdx: func [graph edge] [edge/flags AND CV_SET_ELEM_IDX_MASK]
cvGraphGetVtxCount: func [graph] [graph/active_count]
cvGraphGetEdgeCount: func [graph] [graph/edges/active_count]

CV_GRAPH_VERTEX:        1
CV_GRAPH_TREE_EDGE:     2
CV_GRAPH_BACK_EDGE:     4
CV_GRAPH_FORWARD_EDGE:  8
CV_GRAPH_CROSS_EDGE:    16
CV_GRAPH_ANY_EDGE:      30
CV_GRAPH_NEW_TREE:      32
CV_GRAPH_BACKTRACKING:  64
CV_GRAPH_OVER:          -1
CV_GRAPH_ALL_ITEMS:     -1

;flags for graph vertices and edges 
CV_GRAPH_ITEM_VISITED_FLAG:  1 shift/left 30 1
CV_GRAPH_SEARCH_TREE_NODE_FLAG:  1 shift/left 29 1
CV_GRAPH_FORWARD_EDGE_FLAG:      1 shift/left 28 1 
IS_GRAPH_VERTEX_VISITED: func [vtx] [CV_IS_GRAPH_VERTEX_VISITED: vtx/flags and CV_GRAPH_ITEM_VISITED_FLAG] ; vtx is CvGraphVtx!
IS_GRAPH_EDGE_VISITED: func [edge] [CV_IS_GRAPH_EDGE_VISITED: edge/flags and CV_GRAPH_ITEM_VISITED_FLAG]; edge is CvGraphEdge!

CvGraphScanner!: make struct! compose/deep/only [ 
    vtx 			[struct! (first CvGraphVtx!)] ; current graph vertex (or current edge origin) 
    dst     		[struct! (first CvGraphVtx!)] ; current graph edge destination vertex 
    edge   			[struct! (first CvGraphEdge!)]  ; current edge 
    graph      		[struct! (first CvGraph!)] ; the graph 
	stack		    [struct! (first CvSeq!)] ; the graph vertex stack 
    index			[integer!]        ; the lower bound of certainly visited vertices 
    mask 			[integer!]        ; event mask 
]
none 

;Creates new graph scanner
cvCreateGraphScanner: make routine! compose/deep/only [
	graph			[struct! (first CvGraph!)]
	vtx 			[struct! (first CvGraphVtx!)] ;CV_DEFAULT(NULL)
	mask			[integer!];V_DEFAULT(CV_GRAPH_ALL_ITEMS))
	return:			[struct! (first CvGraphScanner!)]
] libopencv "cvCreateGraphScanner"

;Releases graph scanner
cvReleaseGraphScanner: make routine! compose/deep/only [
	scanner		[struct! (first CvGraphScanner!)] ; CvGraphScanner** address??
] libopencv "cvReleaseGraphScanner"

; Get next graph element 
cvNextGraphItem: make routine! compose/deep/only [
	scanner		[struct! (first CvGraphScanner!)]
	return:		[integer!]
] libopencv "cvNextGraphItem"

;Creates a copy of graph
cvCloneGraph: make routine! compose/deep/only [
	graph		[struct! (first CvGraph!)]
	storage		[struct! (first CvMemStorage!)]
	return: 	[struct! (first CvGraph!)]
] libopencv "cvCloneGraph"

;/****************************************************************************************\
;*                                     Drawing                                            *
;\****************************************************************************************/

;/****************************************************************************************\
;*       Drawing functions work with images/matrices of arbitrary type.                   *
;*       For color images the channel order is BGR[A]                                     *
;*       Antialiasing is supported only for 8-bit image now.                              *
;*       All the functions include parameter color that means rgb value (that may be      *
;*       constructed with CV_RGB macro) for color images and brightness                   *
;*       for grayscale images.                                                            *
;*       If a drawn figure is partially or completely outside of the image, it is clipped.*
;\****************************************************************************************/

CV_RGB: func [r g b] [cvScalar b g r 0 ]
CV_FILLED:	 -1
CV_AA:		 16
;Draws 4-connected, 8-connected or antialiased line segment connecting two points

;ATTENTION RGB values are bit: -128 .. 127 and not 0..2550
; we use a rebol func to transform

tocvRGB: func [color [tuple!] /local r g b a]
[ val: color/1 / 255 either val > 0.5 [r: val * 127] [r: - (val * 128)]
  val: color/2 / 255 either val > 0.5 [g: val * 127] [g: - (val * 128)]
  val: color/3 / 255 either val > 0.5 [b: val * 127] [b: - (val * 128)]
  n: to-integer length? color
  if (n = 3) [a: 0.0]
  if (n = 4 ) [ val: color/4 / 255 either val > 0.5 [a: val * 127] [a: - (val * 128)]]
  return reduce [r g b a]
]


cvLine: make routine! compose/deep/only [
	img				[struct! (first CvArr*)]
	pt1_x			[integer!]
	pt1_y 			[integer!]
	pt2_x 			[integer!]
	pt2_y 			[integer!]
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
	thickness		[integer!] ;
	line_type		[integer!] ;CV_DEFAULT(8)
	_shift			[integer!] ;CV_DEFAULT(0)
] libopencv "cvLine"

; rebol  version
rcvLine: func [ptr start [pair!] end [pair!] color [tuple!] thickness [integer!] lineType [integer!] offset [integer!] 
/local r g b a]
[ tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
  cvLine ptr start/x start/y end/x end/y r g b a thickness lineType offset
]



;Draws a rectangle given two opposite corners of the rectangle (pt1 & pt2),
;if thickness<0 (e.g. thickness == CV_FILLED), the filled box is drawn 

cvRectangle: make routine! compose/deep/only [
	img				[struct! (first CvArr*)]
	pt1_x			[integer!]
	pt1_y 			[integer!]
	pt2_x 			[integer!]
	pt2_y 			[integer!]
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
	thickness		[integer!]
	line_type		[integer!];CV_DEFAULT(8)
	_shift			[integer!];CV_DEFAULT(0)
] libopencv "cvRectangle"

;rebol version
rcvRectangle: func [ptr start [pair!] end [pair!] color [tuple!] thickness [integer!] lineType [integer!] offset [integer!]
/local r g b a]
[ tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
  cvRectangle ptr start/x start/y end/x end/y r g b a thickness lineType offset
]


;Draws a circle with specified center and radius.
;Thickness works in the same way as with cvRectangle

cvCircle: make routine! compose/deep/only [
	img				[struct! (first CvArr*)]
	center_x		[integer!]
	center_y 		[integer!]
	radius			[integer!]
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
	thickness		[integer!];CV_DEFAULT(1)
	line_type		[integer!];CV_DEFAULT(8)
	_shift			[integer!];CV_DEFAULT(0)
] libopencv "cvCircle"

;rebol version
rcvCircle: func [ptr center [pair!] radius [integer!] color [tuple!] thickness [integer!] lineType [integer!] offset [integer!] 
/local r g b a] 
[ tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
  cvCircle ptr center/x center/y radius r g b a thickness lineType offset
]

{Draws ellipse outline, filled ellipse, elliptic arc or filled elliptic sector,
depending on <thickness>, <start_angle> and <end_angle> parameters. The resultant figure
is rotated by <angle>. All the angles are in degrees}

cvEllipse: make routine! compose/deep/only [
	img				[struct! (first CvArr*)]
	center_x		[integer!]
	center_y 		[integer!]
	width			[integer!]
	height			[integer!]
	angle			[decimal!]
	start_angle		[decimal!]
	end_angle		[decimal!]
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
	thickness		[integer!];CV_DEFAULT(1)
	line_type		[integer!];CV_DEFAULT(8)
	_shift			[integer!];CV_DEFAULT(0)
] libopencv "cvEllipse"
; rebol version
rcvEllipse: func [ptr center [pair!] axes [pair!] angle [number!] start_angle [number!] end_angle [number!] color [tuple!] thickness [integer!] lineType [integer!] offset [integer!]
/local r g b a] 
[  tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
  cvEllipse ptr center/x center/y axes/x axes/y to-decimal angle to-decimal start_angle to-decimal end_angle r g b a thickness lineType offset
]

cvEllipseBox: func [ptr center [pair!] size [pair!] angle [number!] color [tuple!] thickness [integer!] lineType [integer!] offset [integer!]
/local r g b a axes]
[ tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
  axes: make struct! cvSize! none
  axes/width: first size 
  axes/height: second size
  cvEllipse ptr center/x center/y axes/width axes/height to-decimal angle 0 360 r g b a thickness lineType offset]


; Fills convex or monotonous polygon.
cvFillConvexPoly: make routine! compose/deep/only [ 
	img				[struct! (first CvArr*)]
	pts				[integer!] ; pointer to array of points 
	npts			[integer!] ; nb of vertices
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
	line_type		[integer!];CV_DEFAULT(8)
	_shift			[integer!];CV_DEFAULT(0)
] libopencv "cvFillConvexPoly"	

; rebol version using a block of pairs as parameter
rcvFillConvexPoly: func [ptr pts [block!] color [tuple!] lineType [integer!] offset [integer!] 
/local r g b a ]
[
	tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
	;how to pass an array of coords as a pointer to cvFillConvexPoly
	npts: length? pts 
	points: make binary! 2 * npts * sizeof 'integer!
	; y first then x
	foreach p pts [insert points convert second p insert points convert first p]
	&pts:  address? points ; pointer to array
	cvFillConvexPoly  ptr &pts npts r g b a  lineType offset
]

;Fills an area bounded by one or more arbitrary polygons
cvFillPoly: make routine! compose/deep/only [ 
	img				[struct! (first CvArr*)]
	pts				[integer!] ; pointer to array of array of points 
	npts			[integer!] ;pointer nb of points by polygons
	contours		[integer!] ; nb of polygons to draw
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
	line_type		[integer!];CV_DEFAULT(8)
	_shift			[integer!];CV_DEFAULT(0)
] libopencv "cvFillPoly"


; cvFillPoly is rather complicated to use with rebol: pointer pf pointers! 
; rcvFillPoly does the same job with a repetition of cvFillConvexPoly according to the  numbers of polygons to be drawn
rcvFillPoly:  func [ptr polygons [block!] color [tuple!] lineType [integer!] offset [integer!] 
/local r g b a]
[
	tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
	foreach p polygons [
			npts: length? p ; nbr of vertices in the polygon 
			points: make binary! 2 * npts * sizeof 'integer!
			foreach pp p [insert points convert second pp insert points convert first pp] ; make the array of coordinates in binary
			&pts:  address? points ; pointer to array of binary (returns an integer!) 
			cvFillConvexPoly  ptr &pts npts r g b a  lineType offset ; draw polygon
	]
]


;Draws one or more polygonal curves
cvPolyLine: make routine! compose/deep/only [ 
	img				[struct! (first CvArr*)]
	pts				[integer!] ; pointer to array of points 
	npts			[integer!] ;nb of vertices
	contours		[integer!] ; nb of polygons
	is_closed		[integer!] ; 0 or 1 to close the polygon
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
	thickness 		[integer!];CV_DEFAULT(1)
	line_type		[integer!];CV_DEFAULT(8)
	_shift			[integer!];CV_DEFAULT(0)
] libopencv "cvPolyLine"

; idem uses cvLine to make polygons
rcvPolyLine: func [ptr lines [block!] isClosed [logic!] color [tuple!] thickness [integer!] lineType [integer!] offset [integer!] 
/local r g b a ]
[
	tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
	nbofLines: to-integer length? lines
	; transforms pair! in 2 integers
	coords: copy []
	foreach l lines [
		bloc: copy []
		val: first l
		append bloc val/x
		append bloc val/y
	    append/only coords bloc
	]
	
	; if isClosed closes the polygon 
	either isClosed [lastPoint: nbofLines] [lastPoint: nbofLines - 1] 
	
	; get starting and ending point for each lines
	for i 1 lastPoint 1 [
		val1: coords/(i)
		val2: coords/(i + 1)
		if i = nbofLines [val2: coords/1]
		cvLine ptr first val1 second val1 first val2 second val2 r g b a thickness lineType offset
	]
]


;Clips the line segment connecting *pt1 and *pt2 by the rectangular window (0<=x<img_size.width, 0<=y<img_size.height).
cvClipLine: make routine! compose/deep/only [ 
	width 		[integer!];x CvSize
	height 		[integer!]; y CvSize
	*pt1		[integer!]; pointer to cvPoint
	*pt2		[integer!] ; pointer to cvPoint
	return:		[integer!]
] libopencv "cvClipLine"

; OK returns 0
rcvClipLine: func [size [pair!] pt1 [pair!] pt2 [pair!]] [
	w: first size
	h: second size
	; creates pointers to cvPoint
	*pt1: make binary! 2 * sizeof 'integer!
	insert *pt1 convert second pt1 insert *pt1 convert first pt1
	&pt1: address? *pt1
	*pt2: make binary! 2 * sizeof 'integer!
	insert *pt2 convert second pt2 insert *pt2 convert first pt2
	&pt2: address? *pt2
	ret: cvClipLine w h &pt1 &pt2
	return ret
]

{Initializes line iterator. Initially, line_iterator->ptr will point
to pt1 (or pt2, see left_to_right description) location in the image.
Returns the number of pixels on the line between the ending points}

cvInitLineIterator: make routine! compose/deep/only [ 
	img				[struct! (first CvArr*)]
	pt1_x			[integer!]
	pt1_y 			[integer!]
	pt2_x 			[integer!]
	pt2_y 			[integer!]
	line_iterator   [struct! (first CvLineIterator!)] ; pointer 
	connectivity	[integer!] ;CV_DEFAULT(8)
	left_to_right	[integer!] ;CV_DEFAULT(0)
	return:			[integer!]
] libopencv "cvInitLineIterator"


;Moves iterator to the next line point 
CV_NEXT_LINE_POINT: func [line_iterator] 
[
	either line_iterator/err < 0 [_line_iterator_mask: -1] [_line_iterator_mask: 0]
	line_iterator/err: line_iterator/err + line_iterator/delta + (line_iterator/plus_delta AND _line_iterator_mask)
	line_iterator/ptr: line_iterator/ptr + line_iterator/minus_step + (line_iterator/plus_step AND _line_iterator_mask)
]

;/basic font types */
 CV_FONT_HERSHEY_SIMPLEX:         0
 CV_FONT_HERSHEY_PLAIN:           1
 CV_FONT_HERSHEY_DUPLEX:          2
 CV_FONT_HERSHEY_COMPLEX:         3 
 CV_FONT_HERSHEY_TRIPLEX:         4
 CV_FONT_HERSHEY_COMPLEX_SMALL:   5
 CV_FONT_HERSHEY_SCRIPT_SIMPLEX:  6
 CV_FONT_HERSHEY_SCRIPT_COMPLEX:  7
 
; font flags 
CV_FONT_ITALIC:                 16  

CV_FONT_VECTOR0:    CV_FONT_HERSHEY_SIMPLEX

; Font structure 
CvFont!: make struct! 
[
    font_face 	[integer!]; /* =CV_FONT_* */
    ascii 		[integer!]; int* /* font data and metrics */
    greek		[integer!]; int*
    cyrillic	[integer!]; int*
    hscale 		[decimal!]
    vscale		[decimal!]
    shear		[decimal!] ; /* slope coefficient: 0 - normal, >0 - italic */
    thickness	[integer!] ; /* letters thickness */
    dx			[decimal!]; /* horizontal interval between letters */
    line_type	[integer!];
] none

;Initializes font structure used further in cvPutText

cvInitFont: make routine! compose/deep/only [ 
	*font				[struct! (first CvFont!)] ; pointer to fonts
	font_face			[integer!]
	hscale				[decimal!]
	vscale				[decimal!]
	shear				[decimal!]; CV_DEFAULT(0) ;italic
	thickness			[integer!]; CV_DEFAULT(1)
	line_type			[integer!];CV_DEFAULT(8))
] libopencv "cvInitFont" 

cvFont: func [scale thickness]
[
    font: make struct! CvFont! none;
    cvInitFont font CV_FONT_HERSHEY_PLAIN scale scale 0 thickness CV_AA
    return font;
]

; Renders text stroke with specified font and color at specified location. CvFont should be initialized with cvInitFont
cvPutText: make routine! compose/deep/only [  
	img				[struct! (first CvArr*)]
	text			[string!]
	orgx			[integer!]
	orgy			[integer!]
	font			[struct! (first CvFont!)]; pointer
	r				[decimal!]
	g				[decimal!]
	b				[decimal!]
	a				[decimal!]
] libopencv "cvPutText"

rcvPutText: func [ptr text [string!] org [pair!] font color [tuple!] 
/local r g b a ]
[
	tmp: tocvRGB color r: tmp/3 g: tmp/2 b: tmp/1 a: tmp/4
	cvPutText ptr text org/x org/y font r g b a
]

;Calculates bounding box of text stroke (useful for alignment)
cvGetTextSize: make routine! compose/deep/only [  
	text			[string!]
	;font			[integer!]; pointer
	font			[struct! (first CvFont!)]; pointer
	text_size		[integer!]; pointer to cvSize*
	baseline		[integer!]; pointer
	
] libopencv "cvGetTextSize" 
