#! /usr/bin/rebol
REBOL [
	Title:		"OpenCV Binding: highgui"
	Author:		"Franois Jouen"
	Rights:		"Copyright (c) 2012 Franois Jouen. All rights reserved."
	License: {
		Redistribution and use in source and binary forms, with or without modification,
		are permitted provided that the following conditions are met:

		    * Redistributions of source code must retain the above copyright notice,
		      this list of conditions and the following disclaimer.
		    * Redistributions in binary form must reproduce the above copyright notice,
		      this list of conditions and the following disclaimer in the documentation
		      and/or other materials provided with the distribution.

		THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
		ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
		FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
		DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
		SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
		CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
		OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
		OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	}
]

do %cxtypes.r
;do %cxcore.r

libopencv: load/library %/Library/Frameworks/OpenCV.framework/Versions/A/OpenCV
;windows: highgui.dll





; not used in rebol version : make the struct! directly in the routine !
;CvVideoWriter!: make struct! [ ptr [integer!]] none
;CvCapture!: make struct! [ ptr [integer!]] [0]
;CvArr!: func [] [] ; metatype for images used as a function parameter



set 'IPL_DEPTH_8U 1
; windows 

set 'CV_WINDOW_AUTOSIZE 1
; mouse events 
 set 'CV_EVENT_MOUSEMOVE 0
 set 'CV_EVENT_LBUTTONDOWN 1
 set 'CV_EVENT_RBUTTONDOWN 2
 set 'CV_EVENT_MBUTTONDOWN 3
 set 'CV_EVENT_LBUTTONUP 4
 set 'CV_EVENT_RBUTTONUP 5
 set 'CV_EVENT_MBUTTONUP 6
 set 'CV_EVENT_LBUTTONDBLCLK 7
 set 'CV_EVENT_RBUTTONDBLCLK 8
 set 'CV_EVENT_MBUTTONDBLCLK 9
 set 'CV_EVENT_FLAG_LBUTTON 1
 set 'CV_EVENT_FLAG_RBUTTON 2
 set 'CV_EVENT_FLAG_MBUTTON 4
 set 'CV_EVENT_FLAG_CTRLKEY 8
 set 'CV_EVENT_FLAG_SHIFTKEY 16
 set 'CV_EVENT_FLAG_ALTKEY 32
;flags when loading image
set 'CV_LOAD_IMAGE_UNCHANGED  -1 ;8bit, color or not
set 'CV_LOAD_IMAGE_GRAYSCALE   0 ;8bit, gray
set 'CV_LOAD_IMAGE_COLOR       1 ; ?, color
set 'CV_LOAD_IMAGE_ANYDEPTH    2 ;any depth, ? 
set 'CV_LOAD_IMAGE_ANYCOLOR    4 ;?, any color
; convert one image to another 
set 'CV_CVTIMG_FLIP      1
set 'CV_CVTIMG_SWAP_RB   2 
set 'CV_DEFAULT			 0
; playing with camera
set 'CV_CAP_ANY      0    ; autodetect
set 'CV_CAP_MIL      100   ; MIL proprietary drivers
set 'CV_CAP_VFW      200   ; platform native
set 'CV_CAP_V4L      200
set 'CV_CAP_V4L2     200
set 'CV_CAP_FIREWARE 300   ; IEEE 1394 drivers
set 'CV_CAP_IEEE1394 300
set 'CV_CAP_DC1394   300
set 'CV_CAP_CMU1394  300
set 'CV_CAP_STEREO   400   ;TYZX proprietary drivers
set 'CV_CAP_TYZX     400
set 'CV_TYZX_LEFT    400
set 'CV_TYZX_RIGHT   401
set 'CV_TYZX_COLOR   402
set 'CV_TYZX_Z       403
set 'CV_CAP_QT       500   ; QuickTime
; capture properties
set 'CV_CAP_PROP_POS_MSEC       0
set 'CV_CAP_PROP_POS_FRAMES     1
set 'CV_CAP_PROP_POS_AVI_RATIO  2
set 'CV_CAP_PROP_FRAME_WIDTH    3
set 'CV_CAP_PROP_FRAME_HEIGHT   4
set 'CV_CAP_PROP_FPS            5
set 'CV_CAP_PROP_FOURCC         6
set 'CV_CAP_PROP_FRAME_COUNT    7 
set 'CV_CAP_PROP_FORMAT         8
set 'CV_CAP_PROP_MODE           9
set 'CV_CAP_PROP_BRIGHTNESS    10
set 'CV_CAP_PROP_CONTRAST      11
set 'CV_CAP_PROP_SATURATION    12
set 'CV_CAP_PROP_HUE           13
set 'CV_CAP_PROP_GAIN          14
set 'CV_CAP_PROP_CONVERT_RGB   15

CV_FOURCC: func [
	c1 [integer!] 
	c2 [integer!]
	c3 [integer!]
	c4 [integer!]
	/local v1 v2 v3 v'][
	v1: c1 and 255 
	v2: (c2 and 255) << 8  
	v3: (c3 and 255) << 16
	v4: (c4 and 255) << 24
	reduce [v1 + v2 + V3 + V4]
]

set 'CV_FOURCC_PROMPT -1  ; Open Codec Selection Dialog (Windows only) */
set 'CV_FOURCC_DEFAULT -1 ; Use default codec for specified filename (Linux only) */


cvInitSystem: make routine! [
;this function is used to set some external parameters in case of X Window
	argc [integer!]
	char** [string!]
	return: [integer!]
] libopencv "cvInitSystem"

cvStartWindowThread: make routine! [
;Start a separated window thread that will manage mouse events
	return: [integer!]
] libopencv "cvStartWindowThread"


cvNamedWindow: make routine! [
; create window
	name [string!]
	flags [integer!]
	return: [integer!]
] libopencv "cvNamedWindow"

cvDestroyWindow: make routine! [
;destroy window and all the trackers associated with it
	name [string!]
	return:
] libopencv "cvDestroyWindow"

cvDestroyAllWindows: make routine! [
;destroy all windows and all the trackers associated with it
	return:
] libopencv "cvDestroyAllWindows"

;resize/move window
cvResizeWindow: make routine! [
;resize window
	name [string!]
	width [integer!]
	height [integer!]
	return:
] libopencv "cvResizeWindow"

cvMoveWindow: make routine! [
;move window
	name [string!]
	x [integer!]
	y [integer!]
	return:
] libopencv "cvMoveWindow"

cvGetWindowHandle: make routine! [
;get native window handle (HWND in case of Win32 and Widget in case of X Window
	name [string!]
	return: [integer!]
] libopencv "cvGetWindowHandle"

cvGetWindowName: make routine! [
;get name of highgui window given by its native handle
	name [string!]
	return: [string!]
] libopencv "cvGetWindowName"

; cvCreateImage is defined in cxcore 

cvShowImage: make routine! compose/deep/only [
;display image within window (highgui windows remember their content)
	name [string!]
	image [struct! (first CvArr*)]
	;return: [none]
] libopencv "cvShowImage"

cvCreateTrackbar: make routine!  compose/deep/only [
;create trackbar and display it on top of given window, set callback 
	trackbar_name [string!]
	window_name [string!]
	;value [struct! [(first int*)]]
	value [integer!] ; integer seems to be sufficient ! considered as pointer
	count [integer!]
	on_change [callback [int]]; can be null but avoid: pbs 
	return: [integer!]	
] libopencv "cvCreateTrackbar"


cvGetTrackbarPos: make routine! [
;retrieve trackbar position
	trackbar_name [string!]
	window_name [string!]
	return: [integer!]
] libopencv "cvGetTrackbarPos"

cvSetTrackbarPos: make routine! [
;set trackbar position
	trackbar_name [string!]
	window_name [string!]
	pos [integer!]
	return: [integer!]
] libopencv "cvSetTrackbarPos"

 
cvSetMouseCallback: make routine! [
;  assign callback for mouse events
	window_name [string!]
	on_mouse [callback [int int int int int]] ; pointer sur une fonction avec 5 params
    param	[integer!] ; must be null
] libopencv "cvSetMouseCallback"


cvLoadImage: make routine! compose/deep/only[
{load image from file 
iscolor can be a combination of flags where CV_LOAD_IMAGE_UNCHANGED  overrides the other flags
using CV_LOAD_IMAGE_ANYCOLOR alone is equivalent to CV_LOAD_IMAGE_UNCHANGED
unless CV_LOAD_IMAGE_ANYDEPTH is specified images are converted to 8bit
this function returns a pointer IplImage structure  (see structure.r)
Supported image formats: BMP, DIB, JPEG, JPG, JPE, PNG, PBM, SR, RAS, TIFF, TIF}
	filename [string!]
	flags [integer!]
	return: [struct! (first IplImage!)] ; returns an iplImage structure
] libopencv "cvLoadImage"


cvLoadImageM: make routine! [
; this function returns a pointer CvMat structure  (see structure.r)
	filename [string!]
	iscolor [integer!]
	return: [ struct! [
		type [integer!]
		step [integer!]
		refcount[integer!]
		data [integer!]
		rows [integer!]
		cols [integer!]]
	]
] libopencv "cvLoadImageM"

cvSaveImage: make routine! [
;save image to file
	filename [string!]
	image [struct! [(first CvArr*)]]
	return: [integer!]
] libopencv "cvSaveImage"


cvConvertImage: make routine! compose/deep/only[
{utility function: convert one image to another with optional vertical flip
src and dst are CvArr* i.e a pointer to image. flags:CV_DEFAULT(0)}
	src [struct! [(first CvArr*)]]
	dst [struct! [(first CvArr*)]]
	flags [integer!]
	return:[]
]libopencv "cvConvertImage" 

cvWaitKey: make routine! [
;wait for key event infinitely (delay<=0) or for "delay" milliseconds
	delay [integer!]
	return: [integer!]
] libopencv "cvWaitKey"
 
{****************************************************************************************
*                         Working with Video Files and Cameras                          *
\****************************************************************************************}

cvCreateFileCapture: make routine! [
;start capturing frames from video file 
	filename [string!]
	return:  [struct! [ptr [integer!]]]
] libopencv "cvCreateFileCapture"


cvCreateCameraCapture: make routine![
{start capturing frames from camera: index = camera_index + domain_offset (CV_CAP_*)
Index of the camera to be used. If there is only one camera or it does not matter what camera to use -1 may be passed.}
	index [integer!]
	return:  [struct! [ptr [integer!]]]
] libopencv "cvCreateCameraCapture"

cvGrabFrame: make routine! [
;grab a frame, return 1 on success, 0 on fail. this function is thought to be fast 
	capture [struct! [ptr [integer!]]]
	return: [integer!]
] libopencv "cvGrabFrame" 

 
cvRetrieveFrame: make routine! [
{get the frame grabbed with cvGrabFrame(..) This function may apply some frame processing like 
frame decompression, flipping etc. !!!DO NOT RELEASE or MODIFY the retrieved frame!!}
	capture [struct! [ptr [integer!]]]
	return: [struct! [(first IplImage!)]] 
] libopencv "cvRetrieveFrame" 
   
  
cvQueryFrame: make routine! compose/deep/only[
;Just a combination of cvGrabFrame and cvRetrieveFrame !!!DO NOT RELEASE or MODIFY the retrieved frame!!! 
	capture [struct! [ptr [integer!]]]
	return: [struct! [(first IplImage!) ]]
] libopencv "cvQueryFrame" 
   

cvReleaseCapture: make routine! [
;stop capturing/reading and free resources
	capture [struct! [ptr [integer!]]]
]libopencv "cvReleaseCapture"



cvGetCaptureProperty: make routine! [
;retrieve capture properties
			capture [struct! [ ptr [integer!]]]
			property_id [integer!]
			return: [decimal!]
]libopencv "cvGetCaptureProperty"

cvSetCaptureProperty: make routine! [
; set capture properties
			capture [struct! [ ptr [integer!]]]
			property_id [integer!]
			value [decimal!]
			return: [integer!]
]libopencv "cvSetCaptureProperty"


cvCreateVideoWriter: make routine! compose/deep/only[
;initialize video file writer
			filename [string!]
			fourcc [integer!]
			pfs [decimal!]
			frame-size [struct! [(first cvSize!)]]
			is_color [integer!] ;CV_DEFAULT(1))
			return: [struct! [ ptr [integer!]]] ; CvVideoWriter!
]libopencv "cvCreateVideoWriter"	


cvWriteFrame: "cvWriteFrame" make routine! compose/deep/only[
;write frame to video file
			writer [struct! [ ptr [integer!]]] ;CvVideoWriter!
			image [struct! [(first IplImage!) ]]
] libopencv	"cvWriteFrame"


cvReleaseVideoWriter: make routine! compose/deep/only [
;close video file writer 
			writer [struct! [ ptr [integer!]]] ;CvVideoWriter!
]libopencv "cvReleaseVideoWriter"
